{% load extedu_core %}

var availableColumnsUrl = '{{ component.tab__columns.grid__available_columns.available_columns_url }}';
var dataSourceField = Ext.getCmp('{{ component.field__data_source_name.client_id }}');

var tabPanel = Ext.getCmp('{{ component.tab_panel.client_id }}');

var availableColumnsTree = Ext.getCmp('{{ component.tab__columns.grid__available_columns.client_id }}');
var availableColumnsSM = availableColumnsTree.getSelectionModel();
var addColumnButton = Ext.getCmp('{{ component.tab__columns.grid__available_columns.top_bar.button__add.client_id }}');

var reportColumnsTree = Ext.getCmp('{{ component.tab__columns.grid__report_columns.client_id }}');
var reportColumnsSM = reportColumnsTree.getSelectionModel();
var removeColumnButton = Ext.getCmp('{{ component.tab__columns.grid__report_columns.top_bar.button__remove.client_id }}');


function dataSourceChangeHandler() {
    var dataSourceName = dataSourceField.getValue();
    var loaderParams = availableColumnsTree.loader.baseParams;

    if (dataSourceName) {
        loaderParams.data_source_name = dataSourceName;

        availableColumnsTree.loader.dataUrl = availableColumnsUrl;
        availableColumnsTree.loader.load(
            availableColumnsTree.root,
            function (node) {
                node.loadComplete();
                node.expand();
            }
        );
    } else {
        availableColumnsTree.loader.dataUrl = undefined;
        availableColumnsTree.root.removeAll();
    }

    reportColumnsTree.root.removeAll();
}

dataSourceChangeHandler();
dataSourceField.on('change', dataSourceChangeHandler);


availableColumnsTree.loader.on('beforeload', function(loader, node) {
    loader.baseParams.parent_column_name = node.attributes.full_name;
});


// {# Деактивирует кнопку добавления колонки в отчет, если не выбрана. #}
availableColumnsSM.on('selectionchange', function(sm, node) {
    addColumnButton.setDisabled(!node || node.hasChildNodes());
});


//{# Деактивирует кнопку удаления колонки из отчета, если не выбрана. #}
reportColumnsSM.on('selectionchange', function(sm, node) {
    removeColumnButton.setDisabled(!node);
});


// {# TODO: Скорее всего функции addColumnToReport и removeColumnFromReport #}
// {# TODO: можно объединить в одну. #}

// {# Переносит колонку из грида доступных колонок в грид колонок отчета. #}
function addColumnToReport() {
    var node = availableColumnsSM.getSelectedNode();
    if (!node) {
        Ext.Msg.alert('Внимание!', 'Выберите колонку для добавления в отчет.');
        return;
    }

    if (node.hasChildNodes()) {
        Ext.Msg.alert(
            'Внимание!',
            'Добавлять в отчет можно только колонки без вложенных колонок.'
        );
        return;
    }

    var root = availableColumnsTree.root;
    var nodesHierarchy = [];
    for (var n = node; n !== root; n = n.parentNode) {
        nodesHierarchy.unshift(n);
    }

    var targetNode = reportColumnsTree.root;
    for (var i = 0; i < nodesHierarchy.length; i++) {
        var n = targetNode.findChild('name', nodesHierarchy[i].attributes.name);
        if (!n) {
            n = new Ext.tree.TreeNode(nodesHierarchy[i].attributes);
            targetNode.appendChild(n);
            targetNode.expand();
        }
        targetNode = n;
    }

    node.remove();

    // {# Удаление пустых узлов. #}
    nodesHierarchy.reverse();
    for (var i = 1; i < nodesHierarchy.length; i++) {
        var n = nodesHierarchy[i];
        if (!n.hasChildNodes()) {
            n.remove();
        }
    }
}


// {# Переносит колонку из грида колонок отчета в грид доступных колонок. #}
function removeColumnFromReport() {
    var node = reportColumnsSM.getSelectedNode();
    if (!node) {
        Ext.Msg.alert('Внимание!', 'Выберите колонку для добавления в отчет.');
        return;
    }

    // {# Восстановление всех родительских узлов в гриде доступных колонок. #}
    var nodesHierarchy = [];
    for (var n = node; n !== reportColumnsTree.root; n = n.parentNode) {
        nodesHierarchy.unshift(n);
    }

    var targetNode = availableColumnsTree.root;
    for (var i = 0; i < nodesHierarchy.length; i++) {
        var n = targetNode.findChild('name', nodesHierarchy[i].attributes.name);
        if (!n) {
            n = new Ext.tree.TreeNode(nodesHierarchy[i].attributes);
            targetNode.appendChild(n);
        }
        targetNode = n;
    }

    // {# Восстановление поддерева выбранного узла. #}
    function move(fromNode, toNode) {
        for (var i = 0; i < fromNode.childNodes.length; i++) {
            var fn = fromNode.childNodes[i];
            var tn = new Ext.tree.TreeNode(fn.attributes);

            toNode.appendChild(tn);

            if (fn.hasChildNodes()) {
                move(fn, tn);
            }
        }

        if (fromNode.hasChildNodes()) {
            fromNode.removeAll();
        }
        fromNode.remove();
    }

    move(node, targetNode);

    // {# Удаление пустых узлов. #}
    nodesHierarchy.reverse();
    for (var i = 1; i < nodesHierarchy.length; i++) {
        var n = nodesHierarchy[i];
        if (!n.hasChildNodes()) {
            n.remove();
        }
    }
}


win.on('beforesubmit', function(submit) {
    // {# Добавление в параметры запроса списка колонок отчета. #}
    function getColumns(node) {
        var result = [];
        for (var i = 0; i < node.childNodes.length; i++) {
            var n = node.childNodes[i];
            if (n.hasChildNodes()) {
                Array.prototype.push.apply(result, getColumns(n));
            } else {
                result.push({
                    accessor_name: n.attributes.full_name
                });
            }
        }

        return result;
    }

    submit.params.columns = Ext.util.JSON.encode(
        getColumns(reportColumnsTree.root)
    );
});
// {# ---------------------------------------------------------------------- #}
// {# Заполнение колонок шаблона, которые были созданы ранее. #}


function findNode(rootNode, fullName) {
    var nodeNames = fullName.split('.');

    while (rootNode && nodeNames.length > 0) {
        rootNode = rootNode.findChild('name', nodeNames.shift());
    }

    return rootNode;
}


function initColumn(columnData, parentNode) {
    var node = new Ext.tree.TreeNode({
        name: columnData.name,
        full_name: columnData.full_name,
        title: columnData.title,
        leaf: columnData.leaf,
    });

    parentNode.appendChild(node);

    for (var i = 0; i < columnData.nested.length; i++) {
        initColumn(columnData.nested[i], node);
    }

    var node = findNode(availableColumnsTree.root, columnData.full_name);
    if (node && !node.hasChildNodes()) {
        node.remove();
    }
}


function onWindowShow() {
    var reportTemplateColumns = {{ component.columns|jsonify }};

    for (var i = 0; i < reportTemplateColumns.length; i++) {
        initColumn(reportTemplateColumns[i], reportColumnsTree.root);
    }

    reportColumnsTree.root.cascade(function(node) {node.expand()});

    win.un('show', onWindowShow);
}
win.on('show', onWindowShow);


// {# Удаляет из загруженных колонок те, которые уже добавлены в шаблон. #}
availableColumnsTree.loader.on('load', function(loader, loadedNode) {
    // {# Узел в дереве колонок отчёта, соответствущий загружаемому #}
    var reportColumnNode;
    if (loadedNode.attributes.full_name) {
        reportColumnNode = findNode(
            reportColumnsTree.root, loadedNode.attributes.full_name
        );
    } else {
        reportColumnNode = reportColumnsTree.root;
    }

    if (reportColumnNode) {
        reportColumnNode.eachChild(function(node) {
            if (node.attributes.leaf) {
                var n = loadedNode.findChild(
                    'full_name', node.attributes.full_name
                );
                if (n)
                    n.remove();
            }
        });
    }
});
//{# ----------------------------------------------------------------------- #}

//{# Обработчик Drag'n'Drop #}
function onBeforeDrop(dropObj){
	var selModel = dropObj.source.tree.getSelectionModel(),
		selected = selModel.getSelectedNode(),
		target = dropObj.target,
        append = (dropObj.point==='append');
    dropObj.dropStatus = selected.parentNode === target.parentNode && !append;
    return dropObj.dropStatus
}
